/***********************************************************
 * $Id: fancypcsccontext.h 1694 2015-06-22 16:11:23Z fancylibs $
 *
 * FancyLibs PC/SC wrapper
 *
 * Copyright (C) 2014 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYPCSCCONTEXT_H_
#define FANCYPCSCCONTEXT_H_

#include <fancypcsc.h>
#include <fancyobject.h>

namespace Fancy {

class List;
template<typename T> class ArrayObject;
template<typename T> class Array;

namespace PCSC {

class PCSCCard;

/**
 * A PC/SC context, which allows to enumerate card readers and wait for
 * and connect to cards in a given reader.
 */
class FANCYPCSC_API PCSCContext : virtual public Fancy::Object {

protected:
	PCSCContext();
	virtual ~PCSCContext();

public:

	typedef Fancy::ObjPtr<PCSCContext> ptr_t;

	/**
	 * Connect to PCSC or throw an exception upon error.
	 */
	static ptr_t newInstance();

	/**
	 * Return a list of attached card readers or throw an exception, if no reader is attached.
	 */
	virtual Fancy::ObjPtr<Fancy::List> enumCardReaders() = 0;

	/**
	 * Wait for a card to appear in reader number <code>nreader</code> and return
	 * the ATR of the card.
	 * If the list of readers has not been queried using enumCardReaders() before, this
	 * method internally calls enumCardReaders().
	 * If no card appears* return <code>null</code> after the given number of milliseconds.
	 * This method may throw a PCSCException in some situations, when no card is entered,
	 * so carefully check for exceptions.
	 */
	virtual Fancy::ObjPtr<ArrayObject<int8_t> > waitForCard(size_t nreader, int timeout) = 0;

	/**
	 * Wait for a card to appear in the reader with naem <code>nreader</code> and return
	 * the ATR of the card.
	 * If no card appears* return <code>null</code> after the given number of milliseconds.
	 * This method may throw a PCSCException in some situations, when no card is entered,
	 * so carefully check for exceptions.
	 */
	virtual Fancy::ObjPtr<ArrayObject<int8_t> > waitForCard(const std::string &reader, int timeout) = 0;

	/**
	 * Cancel any waitForCard call, which is running in another thread.
	 */
	virtual void cancelWaitForCard() = 0;

	/**
	 * Connect to a card in reader number <code>nreader</code> and return
	 * a new card object carrying the PCSC card handle.
	 * If the list of readers has not been queried using enumCardReaders() before, this
	 * method internally calls enumCardReaders().
	 */
	virtual Fancy::ObjPtr<PCSCCard> connectCard(size_t nreader, int protocol = PROTOCOL_ANY) = 0;

	/**
	 * Connect to a card in the reader with name <code>reader</code> and return
	 * a new card object carrying the PCSC card handle.
	 */
	virtual Fancy::ObjPtr<PCSCCard> connectCard(const std::string &reader, int protocol = PROTOCOL_ANY) = 0;

	/**
	 * Wait for a card to disappear from reader number <code>nreader</code>.
	 *
	 * If the card disappear within the given number of milliseconds, true is returned.
	 * If the card does not disappear, false is returned.
	 *
	 * This method may throw a PCSCException in some situations, when no card is entered,
	 * so carefully check for exceptions.
	 */
	virtual bool waitForCardRemoval(size_t nreader, int timeout) = 0;

	/**
	 * Wait for a card to appear in the reader with naem <code>nreader</code> and return
	 *
	 * If the card disappear within the given number of milliseconds, true is returned.
	 * If the card does not disappear, false is returned.
	 *
	 * This method may throw a PCSCException in some situations, when no card is entered,
	 * so carefully check for exceptions.
	 */
	virtual bool waitForCardRemoval(const std::string &reader, int timeout) = 0;

};

} /* namespace PCSC */
} /* namespace Fancy */

#endif /* FANCYPCSCCONTEXT_H_ */
